<?php

namespace Spec\Infomaker\Everyware\NewsML;

use Infomaker\Everyware\NewsML\IdfBodyParser;
use Infomaker\Everyware\NewsML\Interfaces\NewsItemPresentation;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\NewsML\Parsers\ContentPartParser;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Infomaker\Everyware\NewsML\Parsers\HtmlEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\IframelyParser;
use Infomaker\Everyware\NewsML\Parsers\ImageGalleryParser;
use Infomaker\Everyware\NewsML\Parsers\ImageParser;
use Infomaker\Everyware\NewsML\Parsers\LinkObjectParser;
use Infomaker\Everyware\NewsML\Parsers\MapEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\ContactInfoParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\PersonParser;
use Infomaker\Everyware\NewsML\Parsers\Metadata\PolygonParser;
use Infomaker\Everyware\NewsML\Parsers\PdfParser;
use Infomaker\Everyware\NewsML\Parsers\ReviewParser;
use Infomaker\Everyware\NewsML\Parsers\SocialEmbedParser;
use Infomaker\Everyware\NewsML\Parsers\TableParser;
use Infomaker\Everyware\NewsML\Parsers\TeaserParser;
use Infomaker\Everyware\NewsML\Parsers\YoutubeParser;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use function Spec\newsML\loadNewsMLFile;

/**
 * @method allOfType(string $string)
 * @method allOfTypes(string[] $array)
 * @method getBody()
 * @method getTypes($type)
 * @method hasType($type)
 * @method firstOfType(string $string)
 * @method firstType()
 * @method removeType(string $string)
 * @method generateContent(NewsItemPresentation $presentation)
 * @method getItemCount()
 */
class IdfBodyParserSpec extends ObjectBehavior
{
    /**
     * @var IdfBodyParser
     */
    private $parser;

    private $objects = [
        'x-im/imagegallery',
        'x-im/content-part',
        'x-im/mapembed',
        'x-im/table',
        'x-im/htmlembed',
        'x-im/image',
        'x-im/pdf',
        'x-im/youtube',
        'x-im/socialembed',
    ];

    private $elements = [
        'headline',
        'preamble',
        'body',
        'dateline',
        'subheadline1',
        'blockquote',
        'pagedateline'
    ];

    function let()
    {
        $objectParsers = [
            SocialEmbedParser::OBJECT_TYPE => new SocialEmbedParser(),
            TeaserParser::OBJECT_TYPE => new TeaserParser(),
            ImageParser::OBJECT_TYPE => new ImageParser(),
            ImageGalleryParser::OBJECT_TYPE => new ImageGalleryParser(),
            TableParser::OBJECT_TYPE => new TableParser(),
            ContentPartParser::OBJECT_TYPE => new ContentPartParser(new ElementParser()),
            ReviewParser::OBJECT_TYPE => new ReviewParser(new ElementParser()),
            LinkObjectParser::OBJECT_TYPE => new LinkObjectParser(),
            HtmlEmbedParser::OBJECT_TYPE => new HtmlEmbedParser(),
            PdfParser::OBJECT_TYPE => new PdfParser(),
            YoutubeParser::OBJECT_TYPE => new YoutubeParser(),
            ContactInfoParser::OBJECT_TYPE => new ContactInfoParser(),
            PersonParser::OBJECT_TYPE => new PersonParser(),
            PolygonParser::OBJECT_TYPE => new PolygonParser(),
            IframelyParser::OBJECT_TYPE => new IframelyParser,
            MapEmbedParser::OBJECT_TYPE => new MapEmbedParser()
        ];

        foreach ($objectParsers as $type => $parser) {
            NewsMLTransformerManager::registerObjectParser($type, $parser);
        }

        $this->beConstructedWith(loadNewsMLFile('idf'));
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(IdfBodyParser::class);
    }

    function it_should_return_a_list_of_paresd_objects()
    {
        $types = array_merge($this->objects, $this->elements);

        foreach ($types as $type) {
            $this->hasType($type)->shouldBe(true);
        }
    }

    function it_should_return_a_list_of_parsed_types()
    {
        $types = array_merge($this->objects, $this->elements);

        foreach ($types as $type) {
            $this->getTypes($type)->shouldContain($type);
        }
    }

    function it_should_be_able_to_filter_on_type()
    {
        $this->allOfType('body')->shouldHaveCount(72);
        $this->allOfType('x-im/image')->shouldHaveCount(9);

        $this->allOfTypes(['body', 'x-im/image'])->shouldHaveCount(72 + 9);
    }

    function it_should_be_able_to_determine_the_positions_of_specified_type()
    {
        $this->allPosOfType('headline')->shouldBe([
            0
        ]);

        $this->allPosOfType('x-im/image')->shouldBe([
            56,
            59,
            61,
            63,
            65,
            67,
            69,
            71,
            75
        ]);
    }

    function it_should_be_able_to_determine_first_occurrence_of_type()
    {
        $parsedBody = $this->getBody();

        $this->firstOfType('headline')->shouldBe($parsedBody[0]);
        $this->firstOfType('x-im/image')->shouldBe($parsedBody[56]);
    }

    function it_should_be_able_to_determine_first_type_encountered_in_body()
    {
        $this->firstType()->shouldBe('headline');
    }

    function it_should_be_able_to_determine_if_body_contains_type()
    {
        $this->hasType('x-im/image')->shouldBe(true);
        $this->hasType('unknownType')->shouldBe(false);
    }

    function it_can_remove_types_from_the_body()
    {
        $originalObjectCount = $this->getWrappedObject()->getItemCount();
        $this->removeType('x-im/image');
        $this->getItemCount()->shouldbe($originalObjectCount - 9);
    }
}
